==> datafield-1.0.22.tar.gz: datafield/src/Plugin/DataField/FieldWidget/YearWidget.php <==
<?php

namespace Drupal\datafield\Plugin\DataField\FieldWidget;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datafield\Plugin\DataField\FieldType\DateItem;
use Drupal\datafield\Plugin\DataFieldWidgetInterface;
use Drupal\datafield\Plugin\Field\FieldType\DataField;

/**
 * Plugin implementation of the 'datetime_default' widget.
 *
 * @FieldWidget(
 *   id = "year",
 *   label = @Translation("Year"),
 *   field_types = {
 *     "date",
 *     "datetime_iso8601",
 *   },
 *   bundle = "year"
 * )
 */
class YearWidget implements DataFieldWidgetInterface {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormElement(&$element, $item = NULL, $setting = []) {
    $element['#type'] = 'datelist';
    $element['#title'] = '';
    $element['#date_part_order'] = ['year'];
    $element['#date_date_format'] = 'Y';
    $element['#date_date_element'] = 'year';
    $element['#date_time_element'] = 'none';
    $element['#date_year_range'] = date('Y') - 20 . ':' . (date('Y') + 10);
    if (!empty($element["#field_settings"]["min"])) {
      $element["#min"] = $element["#field_settings"]["min"];
    }
    if (!empty($element["#field_settings"]["max"])) {
      $element["#max"] = $element["#field_settings"]["max"];
    }
    if (!empty($element["#min"]) && !empty($element["#max"])) {
      $element['#date_year_range'] = implode(':', [
        $element["#min"],
        $element["#max"],
      ]);
    }
    if (empty($element["#default_value"]) && !empty($element["#field_settings"]["default_date_type"])) {
      $relativeDate = DateItem::DEFAULT_VALUE_NOW;
      if ($element["#field_settings"]["default_date_type"] == DateItem::DEFAULT_VALUE_CUSTOM) {
        $relativeDate = $element["#field_settings"]["default_date"];
      }
      if (!empty($relativeDate)) {
        $date = new DrupalDateTime($relativeDate, date_default_timezone_get());
        $format = 'Y';
        $element["#default_value"] = $date->format($format);
      }
    }
    if (!empty($element["#default_value"]) && is_numeric($element["#default_value"])) {
      $element['#default_value'] = DrupalDateTime::createFromFormat('Y', $element['#default_value']);
    }
  }

  /**
   * Creates date object for a given subfield using storage timezone.
   */
  public function createDate($element = []): ?DateTimePlus {
    $date = NULL;
    if ($value = $element["#default_value"]) {
      $storage = $element["#field_storage"];
      $dateType = $storage['datetime_type'];
      $is_date_only = $dateType == 'date';
      if ($dateType == 'timestamp') {
        if ($storage["type"] == 'date') {
          $value = strtotime($value);
        }
        $date = DrupalDateTime::createFromTimestamp($value, DataField::DATETIME_STORAGE_TIMEZONE);
      }
      else {
        $format = $is_date_only ? DataField::DATETIME_DATE_STORAGE_FORMAT : DataField::DATETIME_DATETIME_STORAGE_FORMAT;
        if ($storage["type"] == 'date') {
          switch ($dateType) {
            case 'datetime':
              $format = 'Y-m-d H:i:s';
              break;

            case 'time':
              $format = 'H:i:s';
              break;

            case 'year':
              $format = 'Y';
              break;
          }
        }
        $date = DrupalDateTime::createFromFormat($format, $value, DataField::DATETIME_STORAGE_TIMEZONE);
      }
      if ($is_date_only) {
        $date->setDefaultDateTime();
      }
    }
    return $date;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues($date, array $form, FormStateInterface $form_state) {
    if (empty($date)) {
      return $date;
    }
    $storage_format = 'Y';
    if (is_string($date)) {
      $date = strtotime($date);
    }
    if (is_numeric($date)) {
      $date = DrupalDateTime::createFromTimestamp($date);
    }
    return $date->format($storage_format);
  }

}

==> datafield-2.0.0-rc4.tar.gz: datafield/src/Plugin/DataField/FieldWidget/YearWidget.php <==
<?php

namespace Drupal\datafield\Plugin\DataField\FieldWidget;

use Drupal\Component\Datetime\DateTimePlus;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\datafield\Plugin\DataField\FieldType\DateItem;
use Drupal\datafield\Plugin\DataFieldWidgetInterface;
use Drupal\datafield\Plugin\Field\FieldType\DataField;

/**
 * Plugin implementation of the 'datetime_default' widget.
 *
 * @FieldWidget(
 *   id = "year",
 *   label = @Translation("Year"),
 *   field_types = {
 *     "date",
 *     "datetime_iso8601",
 *   },
 *   bundle = "year"
 * )
 */
class YearWidget implements DataFieldWidgetInterface {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getFormElement(&$element, $item = NULL, $setting = []) {
    $year = !empty($element["#default_value"]) ? (int) $element["#default_value"] : date('Y');
    $startYear = $year - 20;
    $endYear = $year + 10;
    if (in_array($setting["type"], ['datetime_iso8601'])) {
      $element['#type'] = 'select';
      $years = range($startYear, $endYear);
      $element['#options'] = array_combine($years, $years);
      $element['#empty_option'] = $this->t('- None -');
      $element['#process_default_value'] = FALSE;
      return $element;
    }
    $element['#type'] = 'datelist';
    $element['#title'] = '';
    $element['#date_part_order'] = ['year'];
    $element['#date_date_format'] = 'Y';
    $element['#date_date_element'] = 'year';
    $element['#date_time_element'] = 'none';
    $element['#process_default_value'] = FALSE;
    $element['#date_year_range'] = "$startYear:$endYear";
    if (!empty($element["#field_settings"]["min"])) {
      $element["#min"] = $element["#field_settings"]["min"];
    }
    if (!empty($element["#field_settings"]["max"])) {
      $element["#max"] = $element["#field_settings"]["max"];
    }
    if (!empty($element["#min"]) && !empty($element["#max"])) {
      $element['#date_year_range'] = implode(':', [
        $element["#min"],
        $element["#max"],
      ]);
    }
    if (empty($element["#default_value"]) && !empty($element["#field_settings"]["default_date_type"])) {
      $relativeDate = DateItem::DEFAULT_VALUE_NOW;
      if ($element["#field_settings"]["default_date_type"] == DateItem::DEFAULT_VALUE_CUSTOM) {
        $relativeDate = $element["#field_settings"]["default_date"];
      }
      if (!empty($relativeDate)) {
        $date = new DrupalDateTime($relativeDate, date_default_timezone_get());
        $format = 'Y';
        $element["#default_value"] = $date->format($format);
      }
    }
    if (!empty($element["#default_value"]) && is_numeric($element["#default_value"])) {
      $element['#default_value'] = DrupalDateTime::createFromFormat('Y', $element['#default_value']);
    }
  }

  /**
   * Creates date object for a given subfield using storage timezone.
   */
  public function createDate($element = []): ?DateTimePlus {
    $date = NULL;
    if ($value = $element["#default_value"]) {
      $storage = $element["#field_storage"];
      $dateType = $storage['datetime_type'];
      $is_date_only = $dateType == 'date';
      if ($dateType == 'timestamp') {
        if ($storage["type"] == 'date') {
          $value = strtotime($value);
        }
        $date = DrupalDateTime::createFromTimestamp($value, DataField::DATETIME_STORAGE_TIMEZONE);
      }
      else {
        $format = $is_date_only ? DataField::DATETIME_DATE_STORAGE_FORMAT : DataField::DATETIME_DATETIME_STORAGE_FORMAT;
        if ($storage["type"] == 'date') {
          switch ($dateType) {
            case 'datetime':
              $format = 'Y-m-d H:i:s';
              break;

            case 'time':
              $format = 'H:i:s';
              break;

            case 'year':
              $format = 'Y';
              break;
          }
        }
        $date = DrupalDateTime::createFromFormat($format, $value, DataField::DATETIME_STORAGE_TIMEZONE);
      }
      if ($is_date_only) {
        $date->setDefaultDateTime();
      }
    }
    return $date;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues($date, array $form, FormStateInterface $form_state) {
    if (empty($date)) {
      return $date;
    }
    $storage_format = 'Y';
    if (is_string($date)) {
      $date = strtotime($date);
    }
    if (is_numeric($date)) {
      $date = DrupalDateTime::createFromTimestamp($date);
    }
    return $date->format($storage_format);
  }

}
